/**
 * @file
 * VuoList_VuoBlendMode C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoBlendMode_H
#define VuoList_VuoBlendMode_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoBlendMode VuoList_VuoBlendMode
 * A list of @ref VuoBlendMode elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoBlendMode_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoBlendMode elements.
 */
typedef const struct VuoList_VuoBlendMode_struct { void *l; } * VuoList_VuoBlendMode;
/// @}
#endif

/**
 * Creates a new list of @ref VuoBlendMode elements.
 */
VuoList_VuoBlendMode VuoListCreate_VuoBlendMode(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoBlendMode VuoListCopy_VuoBlendMode(const VuoList_VuoBlendMode list);

/**
 * Returns the @ref VuoBlendMode at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoBlendMode VuoListGetValue_VuoBlendMode(const VuoList_VuoBlendMode list, const unsigned long index);

/**
 * Changes the @ref VuoBlendMode at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoBlendMode(const VuoList_VuoBlendMode list, const VuoBlendMode value, const unsigned long index);

/**
 * Inserts the `VuoBlendMode` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoBlendMode(const VuoList_VuoBlendMode list, const VuoBlendMode value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoBlendMode(VuoList_VuoBlendMode list, const VuoBlendMode value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoBlendMode(VuoList_VuoBlendMode list, const VuoBlendMode value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoBlendMode(VuoList_VuoBlendMode list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoBlendMode(VuoList_VuoBlendMode list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoBlendMode(VuoList_VuoBlendMode list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoBlendMode(VuoList_VuoBlendMode list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoBlendMode(VuoList_VuoBlendMode list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoBlendMode(VuoList_VuoBlendMode list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoBlendMode(VuoList_VuoBlendMode list);

/**
 * Removes the `VuoBlendMode` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoBlendMode(VuoList_VuoBlendMode list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoBlendMode(const VuoList_VuoBlendMode list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoBlendMode VuoList_VuoBlendMode_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoBlendMode_getJson(const VuoList_VuoBlendMode value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoBlendMode_getSummary(const VuoList_VuoBlendMode value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoBlendMode VuoList_VuoBlendMode_makeFromString(const char *str);
char * VuoList_VuoBlendMode_getString(const VuoList_VuoBlendMode value);
/// @}

/**
 * @}
 */

#endif
