/**
 * @file
 * VuoList_VuoLeapPointable C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoLeapPointable_H
#define VuoList_VuoLeapPointable_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoLeapPointable VuoList_VuoLeapPointable
 * A list of @ref VuoLeapPointable elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoLeapPointable_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoLeapPointable elements.
 */
typedef const struct VuoList_VuoLeapPointable_struct { void *l; } * VuoList_VuoLeapPointable;
/// @}
#endif

/**
 * Creates a new list of @ref VuoLeapPointable elements.
 */
VuoList_VuoLeapPointable VuoListCreate_VuoLeapPointable(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoLeapPointable VuoListCopy_VuoLeapPointable(const VuoList_VuoLeapPointable list);

/**
 * Returns the @ref VuoLeapPointable at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoLeapPointable VuoListGetValue_VuoLeapPointable(const VuoList_VuoLeapPointable list, const unsigned long index);

/**
 * Changes the @ref VuoLeapPointable at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoLeapPointable(const VuoList_VuoLeapPointable list, const VuoLeapPointable value, const unsigned long index);

/**
 * Inserts the `VuoLeapPointable` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoLeapPointable(const VuoList_VuoLeapPointable list, const VuoLeapPointable value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoLeapPointable(VuoList_VuoLeapPointable list, const VuoLeapPointable value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoLeapPointable(VuoList_VuoLeapPointable list, const VuoLeapPointable value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoLeapPointable(VuoList_VuoLeapPointable list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoLeapPointable(VuoList_VuoLeapPointable list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoLeapPointable(VuoList_VuoLeapPointable list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoLeapPointable(VuoList_VuoLeapPointable list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoLeapPointable(VuoList_VuoLeapPointable list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoLeapPointable(VuoList_VuoLeapPointable list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoLeapPointable(VuoList_VuoLeapPointable list);

/**
 * Removes the `VuoLeapPointable` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoLeapPointable(VuoList_VuoLeapPointable list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoLeapPointable(const VuoList_VuoLeapPointable list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoLeapPointable VuoList_VuoLeapPointable_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoLeapPointable_getJson(const VuoList_VuoLeapPointable value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoLeapPointable_getSummary(const VuoList_VuoLeapPointable value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoLeapPointable VuoList_VuoLeapPointable_makeFromString(const char *str);
char * VuoList_VuoLeapPointable_getString(const VuoList_VuoLeapPointable value);
/// @}

/**
 * @}
 */

#endif
