/**
 * @file
 * VuoList_VuoModifierKey C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoModifierKey_H
#define VuoList_VuoModifierKey_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoModifierKey VuoList_VuoModifierKey
 * A list of @ref VuoModifierKey elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoModifierKey_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoModifierKey elements.
 */
typedef const struct VuoList_VuoModifierKey_struct { void *l; } * VuoList_VuoModifierKey;
/// @}
#endif

/**
 * Creates a new list of @ref VuoModifierKey elements.
 */
VuoList_VuoModifierKey VuoListCreate_VuoModifierKey(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoModifierKey VuoListCopy_VuoModifierKey(const VuoList_VuoModifierKey list);

/**
 * Returns the @ref VuoModifierKey at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoModifierKey VuoListGetValue_VuoModifierKey(const VuoList_VuoModifierKey list, const unsigned long index);

/**
 * Changes the @ref VuoModifierKey at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoModifierKey(const VuoList_VuoModifierKey list, const VuoModifierKey value, const unsigned long index);

/**
 * Inserts the `VuoModifierKey` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoModifierKey(const VuoList_VuoModifierKey list, const VuoModifierKey value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoModifierKey(VuoList_VuoModifierKey list, const VuoModifierKey value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoModifierKey(VuoList_VuoModifierKey list, const VuoModifierKey value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoModifierKey(VuoList_VuoModifierKey list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoModifierKey(VuoList_VuoModifierKey list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoModifierKey(VuoList_VuoModifierKey list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoModifierKey(VuoList_VuoModifierKey list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoModifierKey(VuoList_VuoModifierKey list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoModifierKey(VuoList_VuoModifierKey list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoModifierKey(VuoList_VuoModifierKey list);

/**
 * Removes the `VuoModifierKey` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoModifierKey(VuoList_VuoModifierKey list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoModifierKey(const VuoList_VuoModifierKey list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoModifierKey VuoList_VuoModifierKey_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoModifierKey_getJson(const VuoList_VuoModifierKey value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoModifierKey_getSummary(const VuoList_VuoModifierKey value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoModifierKey VuoList_VuoModifierKey_makeFromString(const char *str);
char * VuoList_VuoModifierKey_getString(const VuoList_VuoModifierKey value);
/// @}

/**
 * @}
 */

#endif
